@@ -151,6 +151,7 @@ typedef enum {
 typedef enum {
 	FILE_TYPE_INVALID,
 	FILE_TYPE_PPTX,
+	FILE_TYPE_PPSX,
 	FILE_TYPE_DOCX,
 	FILE_TYPE_XLSX
 } MsOfficeXMLFileType;
@@ -163,6 +164,7 @@ typedef struct {
 	gboolean preserve_attribute_present;
 	const gchar *uri;
 	GString *content;
+	gboolean title_already_set;
 } MsOfficeXMLParserInfo;
 
 typedef struct {
@@ -185,6 +187,7 @@ static TrackerExtractData data[] = {
 	{ "application/vnd.ms-*",          extract_msoffice },
 	/* MSoffice2007*/
 	{ "application/vnd.openxmlformats-officedocument.presentationml.presentation", extract_msoffice_xml },
+	{ "application/vnd.openxmlformats-officedocument.presentationml.slideshow",    extract_msoffice_xml },
 	{ "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet",         extract_msoffice_xml },
 	{ "application/vnd.openxmlformats-officedocument.wordprocessingml.document",   extract_msoffice_xml },
 	{ NULL, NULL }
@@ -419,7 +422,7 @@ msoffice_convert_and_normalize_chunk (guint8    *buffer,
 	 *
 	 * TODO: Using g_iconv, this extra heap allocation could be
 	 * avoided, re-using over and over again the same output buffer
-	 * for the UTF-8 encoded string 
+	 * for the UTF-8 encoded string
 	 */
 	converted_text = g_convert (buffer,
 	                            chunk_size,
@@ -846,6 +849,14 @@ extract_msword_content (GsfInfile *infile,
 	gsf_input_read (document_stream, 4, tmp_buffer);
 	lcbClx = read_32bit (tmp_buffer);
 
+	/* If we got an invalid or empty length of piece table, just return
+	 * as we cannot iterate over pieces */
+	if (lcbClx <= 0) {
+		g_object_unref (document_stream);
+		g_object_unref (table_stream);
+		return NULL;
+	}
+
 	/* copy the structure holding the piece table into the clx array. */
 	clx = g_malloc (lcbClx);
 	gsf_input_seek (table_stream, fcClx, G_SEEK_SET);
@@ -857,12 +868,20 @@ extract_msword_content (GsfInfile *infile,
 
 	while (TRUE) {
 		if (clx[i] == 2) {
+			/* Nice, a proper structure with contents, no need to
+			 * iterate more. */
 			lcb_piece_table = read_32bit (clx + (i + 1));
 			piece_table = clx + i + 5;
 			piece_count = (lcb_piece_table - 4) / 12;
 			break;
 		} else if (clx[i] == 1) {
-			i = i + 2 + clx[i + 1];
+			/* Oh, a PRC structure with properties of text, not
+			 * real text, so skip it */
+			guint16 GrpPrl_len;
+
+			GrpPrl_len = read_16bit (&clx[i+1]);
+			/* 3 is the length of clxt (1byte) and cbGrpprl(2bytes) */
+			i = i + 3 + GrpPrl_len;
 		} else {
 			break;
 		}
@@ -1649,6 +1668,8 @@ extract_msoffice (const gchar          *uri,
 
 	infile = open_uri (uri);
 	if (!infile) {
+		g_object_unref (file_info);
+		gsf_shutdown ();
 		return;
 	}
 
@@ -1779,6 +1800,7 @@ xml_start_element_handler_text_data (GMarkupParseContext  *context,
 		break;
 
 	case FILE_TYPE_PPTX:
+	case FILE_TYPE_PPSX:
 		info->tag_type = MS_OFFICE_XML_TAG_SLIDE_TEXT;
 		break;
 
@@ -1908,8 +1930,14 @@ xml_text_handler_document_data (GMarkupParseContext  *context,
 		break;
 
 	case MS_OFFICE_XML_TAG_TITLE:
-		tracker_sparql_builder_predicate (info->metadata, "nie:title");
-		tracker_sparql_builder_object_unvalidated (info->metadata, text);
+		if (info->title_already_set) {
+			g_warning ("Avoiding additional title (%s) in MsOffice XML document '%s'",
+			           text, info->uri);
+		} else {
+			info->title_already_set = TRUE;
+			tracker_sparql_builder_predicate (info->metadata, "nie:title");
+			tracker_sparql_builder_object_unvalidated (info->metadata, text);
+		}
 		break;
 
 	case MS_OFFICE_XML_TAG_SUBJECT:
@@ -2020,6 +2048,7 @@ xml_read (MsOfficeXMLParserInfo *parser_info,
 	info.preserve_attribute_present = FALSE;
 	info.uri = parser_info->uri;
 	info.content = parser_info->content;
+	info.title_already_set = parser_info->title_already_set;
 
 	switch (type) {
 	case MS_OFFICE_XML_TAG_DOCUMENT_CORE_DATA: {
@@ -2064,7 +2093,7 @@ xml_read (MsOfficeXMLParserInfo *parser_info,
 		 * using the given context */
 		tracker_gsf_parse_xml_in_zip (parser_info->uri,
 		                              xml_filename,
-		                              context);
+		                              context, NULL);
 		g_markup_parse_context_free (context);
 	}
 
@@ -2102,6 +2131,14 @@ xml_start_element_handler_content_types (GMarkupParseContext  *context,
 		}
 	}
 
+	/* Both part_name and content_type MUST be NON-NULL */
+	if (!part_name || !content_type) {
+		g_message ("Invalid file (part_name:%s, content_type:%s)",
+		           part_name ? part_name : "none",
+		           content_type ? content_type : "none");
+		return;
+	}
+
 	if ((g_ascii_strcasecmp (content_type, "application/vnd.openxmlformats-package.core-properties+xml") == 0) ||
 	    (g_ascii_strcasecmp (content_type, "application/vnd.openxmlformats-officedocument.extended-properties+xml") == 0)) {
 		xml_read (info, part_name + 1, MS_OFFICE_XML_TAG_DOCUMENT_CORE_DATA);
@@ -2116,6 +2153,7 @@ xml_start_element_handler_content_types (GMarkupParseContext  *context,
 		break;
 
 	case FILE_TYPE_PPTX:
+	case FILE_TYPE_PPSX:
 		if ((g_ascii_strcasecmp (content_type, "application/vnd.openxmlformats-officedocument.presentationml.slide+xml") == 0) ||
 		    (g_ascii_strcasecmp (content_type, "application/vnd.openxmlformats-officedocument.drawingml.diagramData+xml") == 0)) {
 			xml_read (info, part_name + 1, MS_OFFICE_XML_TAG_DOCUMENT_TEXT_DATA);
@@ -2181,6 +2219,8 @@ extract_msoffice_xml (const gchar          *uri,
 		file_type = FILE_TYPE_DOCX;
 	} else if (g_ascii_strcasecmp (mime_used, "application/vnd.openxmlformats-officedocument.presentationml.presentation") == 0) {
 		file_type = FILE_TYPE_PPTX;
+	} else if (g_ascii_strcasecmp (mime_used, "application/vnd.openxmlformats-officedocument.presentationml.slideshow") == 0) {
+		file_type = FILE_TYPE_PPSX;
 	} else if (g_ascii_strcasecmp (mime_used, "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet") == 0) {
 		file_type = FILE_TYPE_XLSX;
 	} else {
@@ -2203,6 +2243,7 @@ extract_msoffice_xml (const gchar          *uri,
 	info.preserve_attribute_present = FALSE;
 	info.uri = uri;
 	info.content = g_string_new ("");
+	info.title_already_set = FALSE;
 
 	context = g_markup_parse_context_new (&parser, 0, &info, NULL);
 
@@ -2210,7 +2251,7 @@ extract_msoffice_xml (const gchar          *uri,
 	 * using the given context */
 	tracker_gsf_parse_xml_in_zip (uri,
 	                              "[Content_Types].xml",
-	                              context);
+	                              context, NULL);
 
 	if (info.content) {
 		gchar *content;
