@@ -1,6 +1,6 @@
 /*
- * Copyright (C) 2006, Edward Duffy (eduffy@gmail.com)
- * Copyright (C) 2008, Nokia
+ * Copyright (C) 2006, Edward Duffy <eduffy@gmail.com>
+ * Copyright (C) 2008, Nokia <ivan.frade@nokia.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -18,46 +18,36 @@
  * Boston, MA  02110-1301, USA.
  */
 
+#include "config.h"
+
 #include <string.h>
 
 #include <glib.h>
 
-#include <libtracker-common/tracker-os-dependant.h>
-#include <libtracker-common/tracker-ontology.h>
-#include <libtracker-common/tracker-utils.h>
+#include <libtracker-common/tracker-common.h>
 
 #include <libtracker-extract/tracker-extract.h>
-#include <libtracker-extract/tracker-utils.h>
-
-#define NMM_PREFIX TRACKER_NMM_PREFIX
-#define NFO_PREFIX TRACKER_NFO_PREFIX
-#define NIE_PREFIX TRACKER_NIE_PREFIX
-#define NCO_PREFIX TRACKER_NCO_PREFIX
-#define DC_PREFIX TRACKER_DC_PREFIX
-
-#define RDF_PREFIX TRACKER_RDF_PREFIX
-#define RDF_TYPE RDF_PREFIX "type"
-
-static const gchar *tags[][2] = {
-	{ "TOTEM_INFO_VIDEO_HEIGHT",            "nfo:height"    },
-	{ "TOTEM_INFO_VIDEO_WIDTH",             "nfo:width"     },
-	{ "TOTEM_INFO_FPS",                     "nfo:frameRate"         },
-	{ "TOTEM_INFO_VIDEO_CODEC",             "nfo:codec"     },
-	{ "TOTEM_INFO_VIDEO_BITRATE",           "nfo:averageBitrate"    },
-	{ "TOTEM_INFO_TITLE",                   "nie:title"     },
-	{ "TOTEM_INFO_ARTIST",                  "nco:creator"   },
-	{ "TOTEM_INFO_ALBUM",                   "nmm:musicAlbum"        },
-	{ "TOTEM_INFO_AUDIO_BITRATE",           "nmm:averageBitrate"    },
-	{ "TOTEM_INFO_AUDIO_SAMPLE_RATE",       "nfo:sampleRate"                },
-	{ "TOTEM_INFO_AUDIO_CODEC",             "nfo:codec"             },
-	{ "TOTEM_INFO_AUDIO_CHANNELS",          "nfo:channels"          },
-	{ NULL,                                         NULL                    }
-};
 
 static void extract_totem (const gchar          *uri,
                            TrackerSparqlBuilder *preupdate,
                            TrackerSparqlBuilder *metadata);
 
+static const gchar *tags[][2] = {
+	{ "TOTEM_INFO_VIDEO_HEIGHT",      "nfo:height"         },
+	{ "TOTEM_INFO_VIDEO_WIDTH",       "nfo:width"          },
+	{ "TOTEM_INFO_FPS",               "nfo:frameRate"      },
+	{ "TOTEM_INFO_VIDEO_CODEC",       "nfo:codec"          },
+	{ "TOTEM_INFO_VIDEO_BITRATE",     "nfo:averageBitrate" },
+	{ "TOTEM_INFO_TITLE",             "nie:title"          },
+	{ "TOTEM_INFO_ARTIST",            "nco:creator"        },
+	{ "TOTEM_INFO_ALBUM",             "nmm:musicAlbum"     },
+	{ "TOTEM_INFO_AUDIO_BITRATE",     "nfo:averageBitrate" },
+	{ "TOTEM_INFO_AUDIO_SAMPLE_RATE", "nfo:sampleRate"     },
+	{ "TOTEM_INFO_AUDIO_CODEC",       "nfo:codec"          },
+	{ "TOTEM_INFO_AUDIO_CHANNELS",    "nfo:channels"       },
+	{ NULL, NULL }
+};
+
 static TrackerExtractData data[] = {
 	{ "audio/*", extract_totem },
 	{ "video/*", extract_totem },
@@ -94,7 +84,7 @@ extract_totem (const gchar          *uri,
 	                                      (GDestroyNotify) g_free,
 	                                      (GDestroyNotify) g_free);
 
-	if (tracker_spawn (argv, 10, &totem, NULL)) {
+	if (tracker_spawn (argv, 10, &totem, NULL, NULL)) {
 		gchar **lines, **line;
 		gchar *artist = NULL, *album = NULL;
 		gchar *artist_uri = NULL, *album_uri = NULL;
@@ -114,10 +104,10 @@ extract_totem (const gchar          *uri,
 
 					if (g_strcmp0 (tags[i][0], "TOTEM_INFO_ARTIST") == 0) {
 						artist = g_strdup (value);
-						artist_uri = tracker_uri_printf_escaped ("urn:artist:%s", artist);
+						artist_uri = tracker_sparql_escape_uri_printf ("urn:artist:%s", artist);
 					} else if (g_strcmp0 (tags[i][0], "TOTEM_INFO_ALBUM") == 0) {
 						album = g_strdup (value);
-						album_uri = tracker_uri_printf_escaped ("urn:album:%s", album);
+						album_uri = tracker_sparql_escape_uri_printf ("urn:album:%s", album);
 					} else {
 						g_hash_table_insert (tmp_metadata, g_strdup (tags[i][1]), g_strdup (value));
 					}
